console = {};

console.log = function(log) {
    AdView.log(log);
};

console.debug = console.log;
console.info = console.log;
console.warn = console.log;
console.error = console.log;

window.mraid_init = function() {
    // console.log('mraid_init start');

    var mraid = window.mraid = {};

    // MAST SDK
    mraid.returnResult = function(call) {
        return call().toString();
    };

    // MAST SDK
    mraid.returnInfo = function(call) {
        var info = '';

        var result = call();
        for (property in result) {
            if (info) {
                info += '&';
            }

            info += encodeURIComponent(property) + '='
                    + encodeURIComponent(result[property]);
        }

        // console.log("returning: " + info);
        return info;
    };

    // ///////
    //
    // events
    //

    var EVENTS = mraid.EVENTS = {
        READY : "ready",
        ERROR : "error",
        STATE_CHANGE : "stateChange",
        VIEWABLE_CHANGE : "viewableChange",
        SIZE_CHANGE : "sizeChange",
        EXPOSURE_CHANGE : "exposureChange",
        AUDIO_VOLUME_CHANGE : "audioVolumeChange"
    };

    var listeners = {};

    // MRAID
    mraid.addEventListener = function(event, listener) {
        console.log("addEventListener: " + event);

        var handlers = listeners[event];

        // Create the listeners for the event if not already created
        if (!handlers) {
            listeners[event] = [];
            handlers = listeners[event];
        }

        // Don't add the same listener twice
        for ( var i = 0; i < handlers.length; ++i) {
            if (listener === handlers[i]) {
                return;
            }
        }

        // Add the new listener
        handlers.push(listener);

        if (event == EVENTS.AUDIO_VOLUME_CHANGE) {
            AdView.sendInitialAudioVolumeChangeEvent();
        }
    };

    // MRAID
    mraid.removeEventListener = function(event, listener) {
        console.log("removeEventListener: " + event);

        var handlers = listeners[event];
        if (handlers) {
            // handlers.remove(listener); // not supported
            var idx = handlers.indexOf(listener);
            if (idx != -1) {
                handlers.splice(idx, 1); // remove by splicing out
            } else {
                if (!listener) {
                    // if no specific listener, remove all for the event
                    listeners[event] = [];
                }
            }
        }
    };

    // MRAID / MAST SDK (for size change)
    mraid.fireChangeEvent = function(event, newValue) {
        console.log("fireChangeEvent handler:" + event + " with:" + newValue);

        var handlers = listeners[event];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i](newValue);
            }
        }
    };

    // MRAID / MAST SDK
    mraid.fireSizeChangeEvent = function(width, height) {
        console.log("fireSizeChangeEvent, with:" + width + ", " + height);

        var handlers = listeners[EVENTS.SIZE_CHANGE];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i](width, height);
            }
        }
    };

    // MAST SDK
    mraid.fireErrorEvent = function(message, action) {
        console.log("fireErrorEvent handler:" + message + " action:" + action);

        var handlers = listeners[EVENTS.ERROR];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i](message, action);
            }
        }
    };

    // MAST SDK
    mraid.fireEvent = function(event) {
        console.log("fireEvent handler:" + event);

        var handlers = listeners[event];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i]();
            }
        }
    };

    mraid.fireExposureChangeEvent = function(exposedPercentage, visibleRectangle, occlusionRectangles) {
        console.log("fireExposureChangeEvent handler: exposedPercentage: " + exposedPercentage + " visibleRectangle: " + visibleRectangle +
        " occlusionRectangles: " + occlusionRectangles);

        var handlers = listeners[EVENTS.EXPOSURE_CHANGE];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i](exposedPercentage, visibleRectangle, occlusionRectangles);
            }
        }
    }

    mraid.fireAudioVolumeChangeEvent = function(volumePercentage) {
        console.log("fireAudioVolumeChangeEvent handler: " + volumePercentage);

        var handlers = listeners[EVENTS.AUDIO_VOLUME_CHANGE];
        if (handlers) {
            for ( var i = 0; i < handlers.length; ++i) {
                handlers[i](volumePercentage);
            }
        }
    }

    // ///////
    //
    // version
    //

    // MRAID
    mraid.getVersion = function() {
        console.log("getVersion");

        return ("3.0");
    };

    // ////////
    //
    // supports
    //

    var FEATURES = mraid.FEATURES = {
        SMS : "sms",
        PHONE : "tel",
        EMAIL : "email",
        CALENDAR : "calendar",
        STORE_PICTURE : "storePicture",
        INLINE_VIDEO : "inlineVideo",
        LOCATION : "location",
        VPAID : "vpaid"
    };

    var supportedFeatures = {};

    // MAST SDK
    mraid.setSupports = function(feature, supported) {
        supportedFeatures[feature] = supported;
    };

    // MRAID
    mraid.supports = function(feature) {
        console.log("supports:" + feature);

        return supportedFeatures[feature];
    };

    // ///////
    //
    // state
    //

    var STATES = mraid.STATES = {
        LOADING : "loading",
        DEFAULT : "default",
        EXPANDED : "expanded",
        RESIZED : "resized",
        HIDDEN : "hidden"
    };

    var state = STATES.LOADING;

    // MAST SDK
    mraid.setState = function(newState) {
        var diff = state != newState;

//        alert('State:'+state+' newState:'+newState+' diff:'+diff);

        state = newState;

        if (diff) {
            mraid.fireChangeEvent(EVENTS.STATE_CHANGE, state);
        } else if (state === STATES.RESIZED) {
            // spec says resized -> resized fires an event
            mraid.fireChangeEvent(EVENTS.STATE_CHANGE, state);
        }
    };

    // MRAID
    mraid.getState = function() {
        console.log("getState: value=" + state);

        return state;
    };

    // ///////
    //
    // placementType
    //

    var PLACEMENT_TYPES = mraid.PLACEMENT_TYPES = {
        INLINE : "inline",
        INTERSTITIAL : "interstitial"
    };

    var placementType = PLACEMENT_TYPES.INLINE;

    // MAST SDK
    mraid.setPlacementType = function(newPlacementType) {
        placementType = newPlacementType;
    };

    // MRAID
    mraid.getPlacementType = function() {
        console.log("getPlacementType");

        return placementType;
    };

    // ////////
    //
    // isViewable
    //

    // MRAID
    mraid.isViewable = function() {
        var isViewable = AdView.isViewable();
        console.log("isViewable: " + isViewable);

        return isViewable;
    };

    // ////////
    //
    // open
    //

    // MRAID
    mraid.open = function(url) {
        console.log("open");

        AdView.open(url);
    };

    // ////////
    //
    // close
    //

    // MRAID
    mraid.close = function() {
        console.log("close");

        AdView.close();
    };

    // ////////
    //
    // orientation Properties (expand/interstitial)
    //

    var ORIENTATION_PROPERTIES_FORCE_ORIENTATION = mraid.ORIENTATION_PROPERTIES_FORCE_ORIENTATION = {
        PORTRAIT : "portrait",
        LANDSCAPE : "landscape",
        NONE : "none"
    };

    var orientationProperties = {
        allowOrientationChange : true,
        forceOrientation : ORIENTATION_PROPERTIES_FORCE_ORIENTATION.NONE
    };

    // MRAID
    mraid.setOrientationProperties = function(properties) {
        console.log("setOrientationProperties");

        var writableFields = [ "allowOrientationChange", "forceOrientation" ];

        for (wf in writableFields) {
            var field = writableFields[wf];
            if (properties[field] !== undefined) {
                orientationProperties[field] = properties[field];
            }
        }

        AdView.setOrientationProperties(mraid
                .returnInfo(mraid.getOrientationProperties));
    };

    // MRAID
    mraid.getOrientationProperties = function() {
        console.log("getOrientationProperties");

        return orientationProperties;
    };

    // ////////
    //
    // expand
    //

    var expandProperties = {
        width : 0,
        height : 0,
        useCustomClose : false,
        isModal : true
    };

    // MRAID
    mraid.setExpandProperties = function(properties) {
        console.log("setExpandProperties");
        if (mraid.getState() != "expanded") {
            var writableFields = [ "width", "height", "useCustomClose" ];

            for (wf in writableFields) {
                var field = writableFields[wf];
                if (properties[field] !== undefined) {
                    expandProperties[field] = properties[field];
                }
            }

            AdView.setExpandProperties(mraid.returnInfo(mraid.getExpandProperties));
        }
    };

    // MRAID
    mraid.getExpandProperties = function() {
        console.log("getExpandProperties");

        var retProperties = {
            width : expandProperties.width,
            height : expandProperties.height,
            useCustomClose : expandProperties.useCustomClose,
            isModal : expandProperties.isModal
        };

        return retProperties;
    };

    // MRAID
    mraid.useCustomClose = function(useCustomClose) {
        console.log("Starting from MRAID 3.0 useCustomClose is deprecated.");
        console.log("useCustomClose");

        var property = {
            useCustomClose : useCustomClose
        };

        mraid.setExpandProperties(property);
    };

    // MRAID
    mraid.expand = function(url) {
        // debug:
        // console.log("@@@");
        // console.log("ad content: " + document.body.innerHTML);
        // console.log("@@@");

        console.log("expand");

        AdView.expand(url);
    };

    // ////////
    //
    // resize
    //

    var RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION = mraid.RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION = {
        TOP_LEFT : "top-left",
        TOP_RIGHT : "top-right",
        CENTER : "center",
        BOTTOM_LEFT : "bottom-left",
        BOTTOM_RIGHT : "bottom-right"
    };

    var resizeProperties = {
        width : 0,
        height : 0,
        customClosePosition : RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION.TOP_RIGHT,
        offsetX : 0,
        offsetY : 0,
        allowOffscreen : true
    };

    // MRAID
    mraid.setResizeProperties = function(properties) {
        console.log("setResizeProperties");

        var writableFields = [ "width", "height", "customClosePosition",
                "offsetX", "offsetY", "allowOffscreen" ];

        for (wf in writableFields) {
            if (wf === "customClosePosition") {
                console.log("Starting from MRAID 3.0 customClosePosition for resize properties is deprecated.");
            }
            var field = writableFields[wf];
            if (properties[field] !== undefined) {
                resizeProperties[field] = properties[field];
            }
        }

        AdView.setResizeProperties(mraid
                .returnInfo(mraid.getResizeProperties));
    };

    // MRAID
    mraid.getResizeProperties = function() {
        console.log("getResizeProperties");

        return resizeProperties;
    };

    // ORMMA compatible resize(), also works with mraid no parameter version
    mraid.resize = function(toWidth, toHeight) {
        if (typeof toWidth != "undefined") {
            // console.log("resize with width/height: " + width + ", " +
            // height);

            if (typeof toHeight != "undefined") {
                console.log("resize with width/height: " + toWidth + ", "
                        + toHeight);
                var rp = {
                    width : toWidth,
                    height : toHeight
                };
                mraid.setResizeProperties(rp, "true");
            } else {
                console.log("resize with width/height: " + width);
                var rp = {
                    width : toWidth
                };
                mraid.setResizeProperties(rp, "true");
            }
        } else {
            console.log("mraid style resize");
        }

        AdView.resize();
    };

    // ////////
    //
    // position and size
    //

    var currentOrientation = 0;

    // MRAID
    mraid.getCurrentPosition = function() {
        console.log("getCurrentPosition");

        return JSON.parse(AdView.getCurrentPosition());
    };

    // MRAID
    mraid.getSize = function() {
        console.log("getSize");

        var size = {
            width : currentPosition.width,
            height : currentPosition.height
        };

        return size;
    };

    // MRAID
    mraid.getMaxSize = function() {
        console.log("getMaxSize");

        return JSON.parse(AdView.getMaxSize());
    };

    // MRAID
    mraid.getDefaultPosition = function() {
        console.log("getDefaultPosition");

        return JSON.parse(AdView.getDefaultPosition());
    };

    // MRAID
    mraid.getScreenSize = function() {
        console.log("getScreenSize");

        return JSON.parse(AdView.getScreenSize());
    };

    // MAST SDK
    mraid.setOrientation = function(orientation) {
        currentOrientation = orientation;
    };

    // ////////
    //
    // Store picture

    // MRAID
    mraid.storePicture = function(url) {
        console.log("store picture: " + url);
        AdView.storePicture(url);
    };

    // ////////
    //
    // Create a calendar event

    mraid.createCalendarEvent = function(parameters) {
        if (mraid.supports(FEATURES.CALENDAR)) {
            AdView.createCalendarEntry(JSON.stringify(parameters));
        } else {
            console.log("create calendar event: requested but not supported");
        }
    }

    // ////////
    //
    // Media playback using external player (for inline playback. use HTML5
    // tags)

    // MRAID
    mraid.playVideo = function(url) {
        console.log("play video: " + url);

        AdView.playVideo(url);
    }

    mraid.getLocation = function() {
        console.log("getLocation");
        if (mraid.supports(FEATURES.LOCATION)) {
            return JSON.parse(AdView.getLocation());
        } else {
            console.log("getLocation: requested but not supported");
            return "-1";
        }
    }

    mraid.unload = function() {
        console.log("unload");
        AdView.unload();
    }

    mraid.getCurrentAppOrientation = function() {
        console.log("getCurrentAppOrientation");
        return JSON.parse(AdView.getCurrentAppOrientation());
    }

    // console.log('mraid_init done');
    // end of mraid_init routine
};

if (!window.mraid) {
    window.mraid_init();

    // Set flag indicating mraid library loaded
    AdView.mraidLoaded();
}
